@@ -305,9 +305,12 @@ fn generate_shadow() -> io::Result<()> {
     let reader = BufReader::new(input_file);
     let mut writer = BufWriter::new(output_file);
 
+    let empty_passwords = env::var("virtme_empty_passwords").unwrap_or(0.to_string());
+    let value = if empty_passwords == "1" { "" } else { "!" };
+
     for line in reader.lines() {
         if let Some((username, _)) = line?.split_once(':') {
-            writeln!(writer, "{username}:!:::::::")?;
+            writeln!(writer, "{username}:{value}:::::::")?;
         }
     }
     utils::do_mount(
@@ -586,13 +589,13 @@ fn run_udevd() -> Option<thread::JoinHandle<()>> {
     if let Some(udevd_path) = find_udevd() {
         let handle = thread::spawn(move || {
             disable_uevent_helper();
-            let args: &[&str] = &["--daemon", "--resolve-names=never"];
+            let args: &[&str] = &["--daemon"];
             utils::run_cmd(udevd_path, args);
             log!("triggering udev coldplug");
             utils::run_cmd("udevadm", &["trigger", "--type=subsystems", "--action=add"]);
             utils::run_cmd("udevadm", &["trigger", "--type=devices", "--action=add"]);
             log!("waiting for udev to settle");
-            utils::run_cmd("udevadm", &["settle"]);
+            utils::run_cmd("udevadm", &["settle", "--timeout=300"]);
             log!("udev is done");
         });
         Some(handle)
@@ -656,7 +659,8 @@ fn get_network_handle(
     let network_dev_str = network_dev.unwrap();
     log!("setting up network device {}", network_dev_str);
     Some(thread::spawn(move || {
-        let args = [
+        let script = format!("{}/virtme-udhcpc-script", guest_tools_dir.unwrap());
+        let mut args = vec![
             "udhcpc",
             "-i",
             &network_dev_str,
@@ -664,8 +668,13 @@ fn get_network_handle(
             "-q",
             "-f",
             "-s",
-            &format!("{}/virtme-udhcpc-script", guest_tools_dir.unwrap()),
+            &script,
         ];
+        let hostname_opt_val;
+        if let Ok(hostname) = env::var("virtme_hostname") {
+            hostname_opt_val = format!("hostname:{}", hostname);
+            args.extend(["-x", &hostname_opt_val]);
+        }
         utils::run_cmd("busybox", &args);
     }))
 }
@@ -947,13 +956,23 @@ fn init_xdg_runtime_dir(uid: u32) {
 }
 
 fn run_user_shell(tty_fd: libc::c_int) {
-    let mut args = vec!["-l"];
-    let storage;
+    let mut args = vec![];
+    let cmd;
+
     if let Ok(user) = env::var("virtme_user") {
+        // Check if a shell override is defined.
+        let virtme_shell = env::var("virtme_shell").ok();
+
+        cmd = if let Some(shell) = virtme_shell {
+            format!("su -s {} -- {}", shell, user)
+        } else {
+            format!("su -- {}", user)
+        };
+
         args.push("-c");
-        storage = format!("su -- {user}");
-        args.push(&storage);
+        args.push(&cmd);
     }
+
     print_logo();
     run_shell(tty_fd, &args);
 }
